/* 
This file is part of project maximizing the number of induced C_5 in a graph by
               J. Balogh, P. Hu, B. Lidicky, F. Pfender
 
It solves the program (P') from Claim 8.

Usage:
 g++ mesh-opt.cpp -Wall -O3 -o mesh-opt && ./mesh-opt

If you have OpenMP, you could use
 g++ mesh-opt.cpp -Wall -O3 -o mesh-opt -fopenmp && ./mesh-opt
 
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <istream>
#include <vector>
#include <utility>
#include <assert.h>
#include <cstring>
#include <algorithm>
#include <cstdarg>
#include <cmath>
#include <limits> 


using namespace std;


const double max21 = 0.21;
const int steps = 100; // Number of steps for each Xi in the recursion (you may also use 200)
const double stepsize = max21/steps;


const double Xmax = max21 + 2*stepsize; // Upper bound for Xi

//const double funkyDegree = 0.079; 
const double funkyDegree = 0.0808;

const int funkyDegreeInt = (funkyDegree*steps)/Xmax - 4; // the - 4 is to cover all rounding errors 

const double extra =  5.0*0.21/2.0*0.001/steps;

int main(int argc, char *argv[])
{

	double max_total =  0;

#pragma omp parallel for ordered schedule(dynamic)
    // pick a[1] to be the smallest
	for (int a1 = 0; a1 <= steps; a1++)
    {
        double max = 0;
        int a[6];
        int b[6];
        a[1] = a1;
        
        for (a[2] = a[1]; a[2] <= steps; a[2]++)
        for (a[3] = a[1]; a[3] <= steps; a[3]++)
	    for (a[4] = a[1]; a[4] <= steps; a[4]++)
	    for (a[5] = a[1]; a[5] <= steps; a[5]++)
	{
		for (int i = 1; i <= 5; ++i) b[i] = steps - a[i]; 

		// Kill entires with too few funky edges
    if (
	(b[2]+b[5]+a[3]+a[4] < funkyDegreeInt) ||
	(b[1]+b[3]+a[4]+a[5] < funkyDegreeInt) ||
	(b[2]+b[4]+a[1]+a[5] < funkyDegreeInt) ||
	(b[3]+b[5]+a[1]+a[2] < funkyDegreeInt) ||
	(b[4]+b[1]+a[2]+a[3] < funkyDegreeInt)
	) continue;
		

		int valueInt = a[2]*a[5]*b[3]*b[4] + a[1]*a[3]*b[4]*b[5] + a[2]*a[4]*b[1]*b[5] + a[3]*a[5]*b[1]*b[2] + a[4]*a[1]*b[2]*b[3] + 0.25*(a[1]*a[1]*b[1]*b[1] + a[2]*a[2]*b[2]*b[2] + a[3]*a[3]*b[3]*b[3]+ a[4]*a[4]*b[4]*b[4] + a[5]*a[5]*b[5]*b[5]);
        double value = valueInt*Xmax*Xmax*Xmax*Xmax/(double)(steps*steps*steps*steps); 

		if (max < value)
		{	
			max = value;	
			//cout << value+extra << " " << value << " " << a[1] << " " << a[2] <<  " " << a[3] <<  " " << a[4] <<  " " << a[5] << endl;
		}
	}
#pragma omp ordered
        {        
        cerr << "Done " << a1 << "/" << steps << endl;
        if (max > max_total)
        {
            max_total = max;
            cout << "Max(P)+extra=" << max+extra << " Max(P)=" << max << endl;            
        }
        }
    }
	
    cout << "Discretization of (P'')  is bounded by " << max_total << " and (P'') is bounded by " << max_total + extra << endl;
    
	return 0;
}

